@@ -1,8 +1,8 @@
 (*
  * libbrlapi - A library providing access to braille terminals for applications.
  *
- * Copyright (C) 2002-2008 by
- *   Sbastien Hinderer <Sebastien.Hinderer@ens-lyon.org>
+ * Copyright (C) 2002-2009 by
+ *   Sébastien Hinderer <Sebastien.Hinderer@ens-lyon.org>
  *   Samuel Thibault <Samuel.Thibault@ens-lyon.org>
  * All rights reserved.
  *
@@ -10,9 +10,8 @@
  *
  * This is free software, placed under the terms of the
  * GNU Lesser General Public License, as published by the Free Software
- * Foundation; either version 2.1 of the License,
- * or (at your option) any later version.
- * Please see the file COPYING-API for details.
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -29,8 +28,8 @@ BRLAPI_RELEASE
 BRLAPI_MAJOR
 BRLAPI_MINOR
 BRLAPI_REVISION
-Faire un fichier .in: @VAR@ est remplac par la valeur de VAR
-telle que dfinie dans configure.ac.
+Faire un fichier .in: @VAR@ est remplacé par la valeur de VAR
+telle que définie dans configure.ac.
 *)
 
 (*
@@ -42,7 +41,7 @@ BRLAPI_SOCKETPATH
 BRLAPI_ETCDIR
 BRLAPI_AUTHKEYFILE
 BRLAPI_DEFAUTH
-Idem que prcdemmment
+Idem que précédemmment
 C'est du ni<eau proto. Faut-il binder ?
 *)
 
@@ -69,7 +68,6 @@ val writeArguments_initializer : writeArguments
 type handle
 
 val max_packet_size : int
-val max_keyset_size : int
 
 type errorCode =
   | SUCCESS
@@ -144,7 +142,7 @@ external write :
 BRLAPI_KEYCODE_MAX
 Les constantes pour travailler sur les flags de touches ?
 Les KEY_SYM ?
-Oui: pour tre cohrent avec le fait de fournir le int64
+Oui: pour être cohérent avec le fait de fournir le int64
 Flags: entiers.
 *)
 
