@@ -1,7 +1,7 @@
 /*
  * libbrlapi - A library providing access to braille terminals for applications.
  *
- * Copyright (C) 2005-2008 by
+ * Copyright (C) 2005-2009 by
  *   Alexis Robert <alexissoft@free.fr>
  *   Samuel Thibault <Samuel.Thibault@ens-lyon.org>
  *
@@ -9,9 +9,8 @@
  *
  * This is free software, placed under the terms of the
  * GNU Lesser General Public License, as published by the Free Software
- * Foundation; either version 2.1 of the License,
- * or (at your option) any later version.
- * Please see the file COPYING-API for details.
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -23,3 +22,5 @@
 #include "brlapi.h"
 
 extern const brlapi_writeArguments_t brlapi_writeArguments_initialized;
+extern char *brlapi_protocolException(void);
+extern void brlapi_protocolExceptionInit(brlapi_handle_t *handle);
