@@ -1,22 +1,23 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2003 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2009 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifndef _MESSAGE_H
-#define _MESSAGE_H
+#ifndef BRLTTY_INCLUDED_MESSAGE
+#define BRLTTY_INCLUDED_MESSAGE
 
 #ifdef __cplusplus
 extern "C" {
@@ -25,7 +26,7 @@ extern "C" {
 /* message.h - send a message to Braille and speech */
 
 /* Prototype: */
-extern void message (const char *, short);
+extern int message (const char *mode, const char *string, short flags);
 
 
 /* Flags for the second argument: */
@@ -38,4 +39,4 @@ extern void message (const char *, short);
 }
 #endif /* __cplusplus */
 
-#endif /* _MESSAGE_H */
+#endif /* BRLTTY_INCLUDED_MESSAGE */
