@@ -2,13 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2008 by The BRLTTY Developers.
+ * Copyright (C) 1995-2009 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -87,6 +88,32 @@ deallocateStrings (char **array) {
   free(array);
 }
 
+char *
+joinStrings (const char *const *strings, int count) {
+  char *string;
+  size_t length = 0;
+  size_t lengths[count];
+  int index;
+
+  for (index=0; index<count; index+=1) {
+    length += lengths[index] = strlen(strings[index]);
+  }
+
+  if ((string = malloc(length+1))) {
+    char *target = string;
+
+    for (index=0; index<count; index+=1) {
+      length = lengths[index];
+      memcpy(target, strings[index], length);
+      target += length;
+    }
+
+    *target = 0;
+  }
+
+  return string;
+}
+
 #if defined(HAVE_SYSLOG_H)
 static int syslogOpened = 0;
 
@@ -389,6 +416,12 @@ locatePathName (const char *path) {
   return name;
 }
 
+const char *
+locatePathExtension (const char *path) {
+  const char *name = locatePathName(path);
+  return strrchr(name, '.');
+}
+
 int
 isExplicitPath (const char *path) {
   return locatePathName(path) != path;
@@ -398,14 +431,9 @@ char *
 makePath (const char *directory, const char *file) {
   const int count = 3;
   const char *components[count];
-  int lengths[count];
-  const int last = count - 1;
-  int first = last;
-  int length = 0;
-  int index;
-  char *path;
+  int first = count;
 
-  components[last] = file;
+  components[--first] = file;
   if (!isAbsolutePath(file)) {
     if (directory && *directory) {
       if (!isPathDelimiter(directory[strlen(directory)-1])) components[--first] = "/";
@@ -413,54 +441,28 @@ makePath (const char *directory, const char *file) {
     }
   }
 
-  for (index=first; index<=last; ++index) {
-    length += lengths[index] = strlen(components[index]);
-  }
-
-  if ((path = malloc(length+1))) {
-    char *target = path;
-    for (index=first; index<=last; ++index) {
-      length = lengths[index];
-      memcpy(target, components[index], length);
-      target += length;
-    }
-    *target = 0;
-  }
-  return path;
+  return joinStrings(&components[first], count-first);
 }
 
-void
-fixPath (char **path, const char *extension, const char *prefix) {
-  const unsigned int prefixLength = strlen(prefix);
-  const unsigned int pathLength = strlen(*path);
-  const unsigned int extensionLength = strlen(extension);
-  char buffer[prefixLength + pathLength + extensionLength + 1];
-  unsigned int length = 0;
+char *
+ensureExtension (const char *path, const char *extension) {
+  const char *components[2];
+  int count = 0;
+  const size_t pathLength = strlen(path);
+  const size_t extensionLength = strlen(extension);
 
-  if (prefixLength) {
-    if (!strchr(*path, '/')) {
-      if (strncmp(*path, prefix, prefixLength) != 0) {
-        memcpy(&buffer[length], prefix, prefixLength);
-        length += prefixLength;
-      }
-    }
-  }
-
-  memcpy(&buffer[length], *path, pathLength);
-  length += pathLength;
-
-  if (extensionLength) {
+  components[count++] = path;
+  if (extensionLength)
     if ((pathLength < extensionLength) ||
-        (memcmp(&buffer[length-extensionLength], extension, extensionLength) != 0)) {
-      memcpy(&buffer[length], extension, extensionLength);
-      length += extensionLength;
-    }
-  }
+        (strcmp(&path[pathLength-extensionLength], extension) != 0))
+      components[count++] = extension;
 
-  if (length > pathLength) {
-    buffer[length] = 0;
-    *path = strdupWrapper(buffer);
-  }
+  return joinStrings(components, count);
+}
+
+int
+testPath (const char *path) {
+  return access(path, F_OK) != -1;
 }
 
 int
@@ -566,7 +568,7 @@ getDeviceDirectory (void) {
         const unsigned int rootLength = strlen(root);
         char path[rootLength + directoryLength + 1];
         snprintf(path, sizeof(path), "%s%s", root, directory);
-        if (access(path, F_OK) != -1) {
+        if (testPath(path)) {
           deviceDirectory = strdupWrapper(path);
           goto found;
         }
@@ -683,19 +685,19 @@ isDosDevice (const char *path, const char *prefix) {
 #endif /* ALLOW_DOS_DEVICE_NAMES */
 
 FILE *
-openFile (const char *path, const char *mode) {
+openFile (const char *path, const char *mode, int optional) {
   FILE *file = fopen(path, mode);
   if (file) {
     LogPrint(LOG_DEBUG, "file opened: %s fd=%d", path, fileno(file));
   } else {
-    LogPrint((errno == ENOENT)? LOG_DEBUG: LOG_ERR,
+    LogPrint((optional && (errno == ENOENT))? LOG_DEBUG: LOG_ERR,
              "cannot open file: %s: %s", path, strerror(errno));
   }
   return file;
 }
 
 FILE *
-openDataFile (const char *path, const char *mode) {
+openDataFile (const char *path, const char *mode, int optional) {
   static const char *userDirectory = NULL;
   const char *name = locatePathName(path);
   char *userPath;
@@ -713,15 +715,15 @@ openDataFile (const char *path, const char *mode) {
   if (!*userDirectory) {
     userPath = NULL;
   } else if ((userPath = makePath(userDirectory, name))) {
-    if (access(userPath, F_OK) != -1) {
-      file = openFile(userPath, mode);
+    if (testPath(userPath)) {
+      file = openFile(userPath, mode, optional);
       goto done;
     }
   }
 
-  if (!(file = openFile(path, mode))) {
+  if (!(file = openFile(path, mode, optional))) {
     if (((*mode == 'w') || (*mode == 'a')) && (errno == EACCES) && userPath) {
-      if (makeDirectory(userDirectory)) file = openFile(userPath, mode);
+      if (makeDirectory(userDirectory)) file = openFile(userPath, mode, optional);
     }
   }
 
@@ -778,12 +780,21 @@ readLine (FILE *file, char **buffer, size_t *size) {
  */
 int
 processLines (FILE *file, int (*handler) (char *line, void *data), void *data) {
+  unsigned int lineNumber = 0;
   char *buffer = NULL;
   size_t bufferSize = 0;
 
-  while (readLine(file, &buffer, &bufferSize))
-    if (!handler(buffer, data))
-      break;
+  while (readLine(file, &buffer, &bufferSize)) {
+    char *line = buffer;
+
+    if (!lineNumber++) {
+      static const char utf8ByteOrderMark[] = {0XEF, 0XBB, 0XBF};
+      static const unsigned int length = sizeof(utf8ByteOrderMark);
+      if (strncmp(line, utf8ByteOrderMark, length) == 0) line += length;
+    }
+
+    if (!handler(line, data)) break;
+  }
 
   if (buffer) free(buffer);
   return !ferror(file);
@@ -842,12 +853,14 @@ gettimeofday (struct timeval *tvp, void *tzp) {
 }
 #endif /* gettimeofday */
 
+#if (__MINGW32_MAJOR_VERSION < 3) || ((__MINGW32_MAJOR_VERSION == 3) && (__MINGW32_MINOR_VERSION < 15))
 void
 usleep (int usec) {
   if (usec > 0) {
     approximateDelay((usec+999)/1000);
   }
 }
+#endif /* usleep */
 #endif /* __MINGW32__ */
 
 long int
