@@ -2,13 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2008 by The BRLTTY Developers.
+ * Copyright (C) 1995-2009 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -521,8 +522,19 @@ processBootParameters (
     for (optionIndex=0; optionIndex<info->optionCount; ++optionIndex) {
       const OptionEntry *option = &info->optionTable[optionIndex];
       if ((option->bootParameter) && (option->bootParameter <= count)) {
-        const char *parameter = parameters[option->bootParameter-1];
-        if (*parameter) ensureSetting(info, option, parameter);
+        char *parameter = parameters[option->bootParameter-1];
+
+        if (*parameter) {
+          {
+            char *byte = parameter;
+
+            do {
+              if (*byte == '+') *byte = ',';
+            } while (*++byte);
+          }
+
+          ensureSetting(info, option, parameter);
+        }
       }
     }
 
@@ -649,7 +661,7 @@ processConfigurationFile (
   const char *path,
   int optional
 ) {
-  FILE *file = openDataFile(path, "r");
+  FILE *file = openDataFile(path, "r", optional);
   if (file != NULL) { /* The configuration file has been successfully opened. */
     int processed;
 
@@ -678,11 +690,7 @@ processConfigurationFile (
     LogPrint(LOG_ERR, gettext("file '%s' processing error."), path);
     info->errorCount++;
   } else {
-    int ok = optional && (errno == ENOENT);
-    LogPrint((ok? LOG_DEBUG: LOG_ERR), "%s: %s: %s",
-             gettext("cannot open configuration file"),
-             path, strerror(errno));
-    if (ok) return 1;
+    if (optional && (errno == ENOENT)) return 1;
     info->errorCount++;
   }
   return 0;
