@@ -2,13 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2008 by The BRLTTY Developers.
+ * Copyright (C) 1995-2009 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -162,6 +163,7 @@ struct SerialDeviceStruct {
 
 #ifdef __MINGW32__
   HANDLE fileHandle;
+  int pending;
 #endif /* __MINGW32__ */
 
 #ifdef __MSDOS__
@@ -653,6 +655,8 @@ flowControlProc_InputCts (void *arg) {
     serialSetLineRTS(serial, up);
     serialWaitLineCTS(serial, (up = !up), 0);
   }
+
+  return NULL;
 }
 
 static int
@@ -922,6 +926,7 @@ serialCompareAttributes (const SerialAttributes *attributes, const SerialAttribu
 static int
 serialReadAttributes (SerialDevice *serial) {
 #if defined(__MINGW32__)
+  serial->currentAttributes.DCBlength = sizeof(serial->currentAttributes);
   if (GetCommState(serial->fileHandle, &serial->currentAttributes)) return 1;
   LogWindowsError("GetCommState");
 #elif defined(__MSDOS__)
@@ -1024,19 +1029,33 @@ serialAwaitInput (SerialDevice *serial, int timeout) {
   if (!serialFlushAttributes(serial)) return 0;
 
 #ifdef __MINGW32__
-  COMMTIMEOUTS timeouts = {MAXDWORD, 0, timeout, 0, 0};
-  DWORD bytesRead;
-  int ret;
-  char c;
-  if (!(SetCommTimeouts(serial->fileHandle, &timeouts))) {
-    LogWindowsError("SetCommTimeouts serialAwaitInput");
-    return 0;
-  }
+  if (serial->pending != -1) return 1;
 
-  ret = ReadFile(serial->fileHandle, &c, 0, &bytesRead, NULL);
-  if (!ret)
-    LogWindowsError("ReadFile");
-  return ret;
+  {
+    COMMTIMEOUTS timeouts = {MAXDWORD, 0, timeout, 0, 0};
+    DWORD bytesRead;
+    char c;
+
+    if (!(SetCommTimeouts(serial->fileHandle, &timeouts))) {
+      LogWindowsError("SetCommTimeouts serialAwaitInput");
+      setSystemErrno();
+      return 0;
+    }
+
+    if (!ReadFile(serial->fileHandle, &c, 1, &bytesRead, NULL)) {
+      LogWindowsError("ReadFile");
+      setSystemErrno();
+      return 0;
+    }
+
+    if (bytesRead) {
+      serial->pending = (unsigned char)c;
+      return 1;
+    }
+  }
+  errno = EAGAIN;
+
+  return 0;
 #else /* __MINGW32__ */
   return awaitInput(serial->fileDescriptor, timeout);
 #endif /* __MINGW32__ */
@@ -1054,25 +1073,31 @@ serialReadChunk (
   COMMTIMEOUTS timeouts = {MAXDWORD, 0, initialTimeout, 0, 0};
   DWORD bytesRead;
 
-  if (!(SetCommTimeouts(serial->fileHandle, &timeouts))) {
-    LogWindowsError("SetCommTimeouts serialReadChunk1");
-    setSystemErrno();
-    return 0;
-  }
+  if (serial->pending != -1) {
+    * (unsigned char *) buffer = serial->pending;
+    serial->pending = -1;
+    bytesRead = 1;
+  } else {
+    if (!(SetCommTimeouts(serial->fileHandle, &timeouts))) {
+      LogWindowsError("SetCommTimeouts serialReadChunk1");
+      setSystemErrno();
+      return 0;
+    }
 
-  if (!ReadFile(serial->fileHandle, buffer+*offset, count, &bytesRead, NULL)) {
-    LogWindowsError("ReadFile");
-    setSystemErrno();
-    return 0;
-  }
+    if (!ReadFile(serial->fileHandle, buffer+*offset, count, &bytesRead, NULL)) {
+      LogWindowsError("ReadFile");
+      setSystemErrno();
+      return 0;
+    }
 
-  if (!bytesRead) {
-    errno = EAGAIN;
-    return 0;
+    if (!bytesRead) {
+      errno = EAGAIN;
+      return 0;
+    }
   }
 
   count -= bytesRead;
-  offset += bytesRead;
+  *offset += bytesRead;
   timeouts.ReadTotalTimeoutConstant = subsequentTimeout;
   if (!(SetCommTimeouts(serial->fileHandle, &timeouts))) {
     LogWindowsError("SetCommTimeouts serialReadChunk2");
@@ -1087,7 +1112,7 @@ serialReadChunk (
     }
 
     count -= bytesRead;
-    offset += bytesRead;
+    *offset += bytesRead;
   }
 
   if (!count) return 1;
@@ -1134,7 +1159,7 @@ serialWriteData (
     }
 
     while (left && WriteFile(serial->fileHandle, data, left, &bytesWritten, NULL)) {
-      if (!bytesWritten) return size;
+      if (!bytesWritten) break;
       left -= bytesWritten;
       data += bytesWritten;
     }
@@ -1152,10 +1177,20 @@ serialWriteData (
 static int
 serialGetLines (SerialDevice *serial, SerialLines *lines) {
 #if defined(__MINGW32__)
+  DCB dcb;
   if (!GetCommModemStatus(serial->fileHandle, &serial->linesState)) {
     LogWindowsError("GetCommModemStatus");
     return 0;
   }
+  dcb.DCBlength = sizeof(dcb);
+  if (!GetCommState(serial->fileHandle, &dcb)) {
+    LogWindowsError("GetCommState");
+    return 0;
+  }
+  if (dcb.fRtsControl == RTS_CONTROL_ENABLE)
+    serial->linesState |= SERIAL_LINE_RTS;
+  if (dcb.fDtrControl == DTR_CONTROL_ENABLE)
+    serial->linesState |= SERIAL_LINE_DTR;
 #elif defined(__MSDOS__)
   serial->linesState = serialReadPort(serial, SERIAL_PORT_MSR) & 0XF0;
 #elif defined(TIOCMGET)
@@ -1165,7 +1200,7 @@ serialGetLines (SerialDevice *serial, SerialLines *lines) {
   }
 #else /* get lines */
 #warning getting modem lines not supported on this platform
-  serial->linesState = SERIAL_LINE_CTS | SERIAL_LINE_DSR | SERIAL_LINE_CAR;
+  serial->linesState = SERIAL_LINE_RTS | SERIAL_LINE_CTS | SERIAL_LINE_DTR | SERIAL_LINE_DSR | SERIAL_LINE_CAR;
 #endif /* get lines */
 
   *lines = serial->linesState;
@@ -1176,6 +1211,7 @@ static int
 serialSetLines (SerialDevice *serial, SerialLines high, SerialLines low) {
 #if defined(__MINGW32__)
   DCB dcb;
+  dcb.DCBlength = sizeof(dcb);
   if (GetCommState(serial->fileHandle, &dcb)) {
     if (low & SERIAL_LINE_RTS)
       dcb.fRtsControl = RTS_CONTROL_DISABLE;
@@ -1377,6 +1413,10 @@ serialOpenDevice (const char *path) {
             serial->flowControlRunning = 0;
 #endif /* HAVE_POSIX_THREADS */
 
+#ifdef __MINGW32__
+            serial->pending = -1;
+#endif /* __MINGW32__ */
+
             LogPrint(LOG_DEBUG, "serial device opened: %s: fd=%d",
                      device,
 #ifdef __MINGW32__
@@ -1407,7 +1447,12 @@ serialOpenDevice (const char *path) {
         close(serial->fileDescriptor);
 #endif /* __MINGW32__ */
       } else {
+#ifdef __MINGW32__
+        LogWindowsError("CreateFile");
+        LogPrint(LOG_ERR, "cannot open serial device: %s", device);
+#else /* __MINGW32__ */
         LogPrint(LOG_ERR, "cannot open serial device: %s: %s", device, strerror(errno));
+#endif /* __MINGW32__ */
       }
 
       free(device);
@@ -1547,6 +1592,10 @@ isSerialDevice (const char **path) {
   if (isDosDevice(*path, "COM")) return 1;
 #endif /* ALLOW_DOS_DEVICE_NAMES */
 
-  if (isQualifiedDevice(path, "serial")) return 1;
-  return !isQualifiedDevice(path, NULL);
+  if (!isQualifiedDevice(path, "serial"))
+    if (isQualifiedDevice(path, NULL))
+      return 0;
+
+  if (!**path) *path = FIRST_SERIAL_DEVICE;
+  return 1;
 }
