@@ -2,13 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2008 by The BRLTTY Developers.
+ * Copyright (C) 1995-2009 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -42,6 +43,8 @@ int
 findSharedSymbol (void *object, const char *symbol, void *pointerAddress) {
 #ifdef HAVE_DLOPEN 
   void **address = pointerAddress;
+
+  dlerror(); /* clear any previous error condition */
   *address = dlsym(object, symbol);
 
   {
