@@ -2,13 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2009 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -17,8 +18,6 @@
 
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <errno.h>
 #include <mach/mach.h>
@@ -40,7 +39,7 @@ typedef struct {
   int count;
 } UsbAsynchronousRequest;
 
-typedef struct {
+struct UsbDeviceExtensionStruct {
   IOUSBDeviceInterface182 **device;
   unsigned deviceOpened:1;
 
@@ -51,9 +50,9 @@ typedef struct {
   CFRunLoopRef runloopReference;
   CFStringRef runloopMode;
   CFRunLoopSourceRef runloopSource;
-} UsbDeviceExtension;
+};
 
-typedef struct {
+struct UsbEndpointExtensionStruct {
   UsbEndpoint *endpoint;
   Queue *completedRequests;
 
@@ -63,10 +62,10 @@ typedef struct {
   UInt8 transferMode;
   UInt8 pollInterval;
   UInt16 packetSize;
-} UsbEndpointExtension;
+};
 
 static void
-setErrno (long int result, const char *action) {
+setUnixError (long int result, const char *action) {
 #define SET(to) errno = (to); break;
 #define MAP(from,to) case (from): SET(to)
   switch (result) {
@@ -224,12 +223,12 @@ openDevice (UsbDevice *device, int seize) {
 
     result = (*devx->device)->USBDeviceOpen(devx->device);
     if (result != kIOReturnSuccess) {
-      setErrno(result, "USB device open");
+      setUnixError(result, "USB device open");
       if ((result != kIOReturnExclusiveAccess) || !seize) return 0;
 
       result = (*devx->device)->USBDeviceOpenSeize(devx->device);
       if (result != kIOReturnSuccess) {
-        setErrno(result, "USB device seize");
+        setUnixError(result, "USB device seize");
         return 0;
       }
 
@@ -237,7 +236,7 @@ openDevice (UsbDevice *device, int seize) {
       level = LOG_NOTICE;
     }
 
-    LogPrint(level, "USB device %s: vendor=%04X product=%4X",
+    LogPrint(level, "USB device %s: vendor=%04X product=%04X",
              action, device->descriptor.idVendor, device->descriptor.idProduct);
     devx->deviceOpened = 1;
   }
@@ -246,8 +245,7 @@ openDevice (UsbDevice *device, int seize) {
 }
 
 static int
-unsetInterface (UsbDevice *device) {
-  UsbDeviceExtension *devx = device->extension;
+unsetInterface (UsbDeviceExtension *devx) {
   int ok = 1;
 
   if (devx->interface) {
@@ -260,7 +258,7 @@ unsetInterface (UsbDevice *device) {
           for (pipe=1; pipe<=devx->pipeCount; ++pipe) {
             result = (*devx->interface)->AbortPipe(devx->interface, pipe);
             if (result != kIOReturnSuccess) {
-              setErrno(result, "USB pipe abort");
+              setUnixError(result, "USB pipe abort");
             }
           }
         }
@@ -273,7 +271,7 @@ unsetInterface (UsbDevice *device) {
 
       result = (*devx->interface)->USBInterfaceClose(devx->interface);
       if (result != kIOReturnSuccess) {
-        setErrno(result, "USB interface close");
+        setUnixError(result, "USB interface close");
         ok = 0;
       }
       devx->interfaceOpened = 0;
@@ -293,7 +291,7 @@ isInterface (IOUSBInterfaceInterface190 **interface, UInt8 number) {
 
   result = (*interface)->GetInterfaceNumber(interface, &num);
   if (result != kIOReturnSuccess) {
-    setErrno(result, "USB interface number query");
+    setUnixError(result, "USB interface number query");
   } else if (num == number) {
     return 1;
   }
@@ -302,11 +300,10 @@ isInterface (IOUSBInterfaceInterface190 **interface, UInt8 number) {
 }
 
 static int
-setInterface (UsbDevice *device, UInt8 number) {
-  UsbDeviceExtension *devx = device->extension;
+setInterface (UsbDeviceExtension *devx, UInt8 number) {
   int found = 0;
   IOReturn result;
-  io_iterator_t iterator = NULL;
+  io_iterator_t iterator = 0;
 
   if (devx->interface)
     if (isInterface(devx->interface, number))
@@ -335,7 +332,7 @@ setInterface (UsbDevice *device, UInt8 number) {
                                                  kIOCFPlugInInterfaceID,
                                                  &plugin, &score);
       IOObjectRelease(service);
-      service = NULL;
+      service = 0;
 
       if ((result == kIOReturnSuccess) && plugin) {
         IOUSBInterfaceInterface190 **interface = NULL;
@@ -348,7 +345,7 @@ setInterface (UsbDevice *device, UInt8 number) {
 
         if ((result == kIOReturnSuccess) && interface) {
           if (isInterface(interface, number)) {
-            unsetInterface(device);
+            unsetInterface(devx);
             devx->interface = interface;
             found = 1;
             break;
@@ -357,18 +354,18 @@ setInterface (UsbDevice *device, UInt8 number) {
           (*interface)->Release(interface);
           interface = NULL;
         } else {
-          setErrno(result, "USB interface interface create");
+          setUnixError(result, "USB interface interface create");
         }
       } else {
-        setErrno(result, "USB interface service plugin create");
+        setUnixError(result, "USB interface service plugin create");
       }
     }
     if (!found) LogPrint(LOG_ERR, "USB interface not found: %d", number);
 
     IOObjectRelease(iterator);
-    iterator = NULL;
+    iterator = 0;
   } else {
-    setErrno(result, "USB interface iterator create");
+    setUnixError(result, "USB interface iterator create");
   }
 
   return found;
@@ -395,7 +392,14 @@ usbResetDevice (UsbDevice *device) {
   IOReturn result = (*devx->device)->ResetDevice(devx->device);
   if (result == kIOReturnSuccess) return 1;
 
-  setErrno(result, "USB device reset");
+  setUnixError(result, "USB device reset");
+  return 0;
+}
+
+int
+usbDisableAutosuspend (UsbDevice *device) {
+  errno = ENOSYS;
+  LogError("USB device autosuspend disable");
   return 0;
 }
 
@@ -410,7 +414,7 @@ usbSetConfiguration (
     UInt8 arg = configuration;
     IOReturn result = (*devx->device)->SetConfiguration(devx->device, arg);
     if (result == kIOReturnSuccess) return 1;
-    setErrno(result, "USB configuration set");
+    setUnixError(result, "USB configuration set");
   }
 
   return 0;
@@ -424,7 +428,7 @@ usbClaimInterface (
   UsbDeviceExtension *devx = device->extension;
   IOReturn result;
 
-  if (setInterface(device, interface)) {
+  if (setInterface(devx, interface)) {
     if (devx->interfaceOpened) return 1;
 
     result = (*devx->interface)->USBInterfaceOpen(devx->interface);
@@ -434,12 +438,12 @@ usbClaimInterface (
         devx->interfaceOpened = 1;
         return 1;
       } else {
-        setErrno(result, "USB pipe count query");
+        setUnixError(result, "USB pipe count query");
       }
 
       (*devx->interface)->USBInterfaceClose(devx->interface);
     } else {
-      setErrno(result, "USB interface open");
+      setUnixError(result, "USB interface open");
     }
   }
 
@@ -451,7 +455,8 @@ usbReleaseInterface (
   UsbDevice *device,
   unsigned char interface
 ) {
-  return setInterface(device, interface) && unsetInterface(device);
+  UsbDeviceExtension *devx = device->extension;
+  return setInterface(devx, interface) && unsetInterface(devx);
 }
 
 int
@@ -462,31 +467,21 @@ usbSetAlternative (
 ) {
   UsbDeviceExtension *devx = device->extension;
 
-  if (setInterface(device, interface)) {
-    UInt8 arg = alternative;
-    IOReturn result = (*devx->interface)->SetAlternateInterface(devx->interface, arg);
-    if (result == kIOReturnSuccess) return 1;
-    setErrno(result, "USB alternative set");
-  }
-
-  return 0;
-}
-
-int
-usbResetEndpoint (
-  UsbDevice *device,
-  unsigned char endpointAddress
-) {
-  UsbDeviceExtension *devx = device->extension;
-  UsbEndpoint *endpoint;
-
-  if ((endpoint = usbGetEndpoint(device, endpointAddress))) {
-    UsbEndpointExtension *eptx = endpoint->extension;
+  if (setInterface(devx, interface)) {
     IOReturn result;
+    UInt8 arg;
 
-    result = (*devx->interface)->ClearPipeStallBothEnds(devx->interface, eptx->pipeNumber);
-    if (result == kIOReturnSuccess) return 1;
-    setErrno(result, "USB endpoint reset");
+    result = (*devx->interface)->GetAlternateSetting(devx->interface, &arg);
+    if (result == kIOReturnSuccess) {
+      if (arg == alternative) return 1;
+
+      arg = alternative;
+      result = (*devx->interface)->SetAlternateInterface(devx->interface, arg);
+      if (result == kIOReturnSuccess) return 1;
+      setUnixError(result, "USB alternative set");
+    } else {
+      setUnixError(result, "USB alternative get");
+    }
   }
 
   return 0;
@@ -504,9 +499,9 @@ usbClearEndpoint (
     UsbEndpointExtension *eptx = endpoint->extension;
     IOReturn result;
 
-    result = (*devx->interface)->ClearPipeStall(devx->interface, eptx->pipeNumber);
+    result = (*devx->interface)->ClearPipeStallBothEnds(devx->interface, eptx->pipeNumber);
     if (result == kIOReturnSuccess) return 1;
-    setErrno(result, "USB endpoint clear");
+    setUnixError(result, "USB endpoint clear");
   }
 
   return 0;
@@ -541,7 +536,7 @@ usbControlTransfer (
 
   result = (*devx->device)->DeviceRequestTO(devx->device, &arg);
   if (result == kIOReturnSuccess) return arg.wLenDone;
-  setErrno(result, "USB control transfer");
+  setUnixError(result, "USB control transfer");
   return -1;
 }
 
@@ -573,7 +568,7 @@ usbSubmitRequest (
       result = (*devx->interface)->CreateInterfaceAsyncEventSource(devx->interface,
                                                                    &devx->runloopSource);
       if (result != kIOReturnSuccess) {
-        setErrno(result, "USB interface event source create");
+        setUnixError(result, "USB interface event source create");
         return NULL;
       }
       CFRunLoopAddSource(devx->runloopReference,
@@ -592,7 +587,7 @@ usbSubmitRequest (
                                                      request->buffer, request->length,
                                                      usbAsynchronousRequestCallback, request);
           if (result == kIOReturnSuccess) return request;
-          setErrno(result, "USB endpoint asynchronous read");
+          setUnixError(result, "USB endpoint asynchronous read");
           break;
 
         case kUSBOut:
@@ -601,7 +596,7 @@ usbSubmitRequest (
                                                       request->buffer, request->length,
                                                       usbAsynchronousRequestCallback, request);
           if (result == kIOReturnSuccess) return request;
-          setErrno(result, "USB endpoint asynchronous write");
+          setUnixError(result, "USB endpoint asynchronous write");
           break;
 
         default:
@@ -671,7 +666,7 @@ usbReapResponse (
         response->count = -1;
       }
     } else {
-      setErrno(request->result, "USB asynchronous response");
+      setUnixError(request->result, "USB asynchronous response");
       response->error = errno;
       response->count = -1;
     }
@@ -719,18 +714,18 @@ usbReadEndpoint (
 
       case kIOUSBPipeStalled:
         if (!stalled) {
-          result = (*devx->interface)->ClearPipeStall(devx->interface, eptx->pipeNumber);
+          result = (*devx->interface)->ClearPipeStallBothEnds(devx->interface, eptx->pipeNumber);
           if (result == kIOReturnSuccess) {
             stalled = 1;
             goto read;
           }
 
-          setErrno(result, "USB stall clear");
+          setUnixError(result, "USB stall clear");
           break;
         }
 
       default:
-        setErrno(result, "USB endpoint read");
+        setUnixError(result, "USB endpoint read");
         break;
     }
   }
@@ -757,7 +752,7 @@ usbWriteEndpoint (
                                              (void *)buffer, length,
                                              timeout, timeout);
     if (result == kIOReturnSuccess) return length;
-    setErrno(result, "USB endpoint write");
+    setUnixError(result, "USB endpoint write");
   }
 
   return -1;
@@ -799,7 +794,7 @@ usbReadDeviceDescriptor (UsbDevice *device) {
   return 1;
 
 error:
-  setErrno(result, "USB device descriptor read");
+  setUnixError(result, "USB device descriptor read");
   return 0;
 }
 
@@ -832,7 +827,7 @@ usbAllocateEndpointExtension (UsbEndpoint *endpoint) {
             return 1;
           }
         } else {
-          setErrno(result, "USB pipe properties query");
+          setUnixError(result, "USB pipe properties query");
         }
       }
 
@@ -854,9 +849,7 @@ usbAllocateEndpointExtension (UsbEndpoint *endpoint) {
 }
 
 void
-usbDeallocateEndpointExtension (UsbEndpoint *endpoint) {
-  UsbEndpointExtension *eptx = endpoint->extension;
-
+usbDeallocateEndpointExtension (UsbEndpointExtension *eptx) {
   if (eptx->completedRequests) {
     deallocateQueue(eptx->completedRequests);
     eptx->completedRequests = NULL;
@@ -866,15 +859,14 @@ usbDeallocateEndpointExtension (UsbEndpoint *endpoint) {
 }
 
 void
-usbDeallocateDeviceExtension (UsbDevice *device) {
-  UsbDeviceExtension *devx = device->extension;
+usbDeallocateDeviceExtension (UsbDeviceExtension *devx) {
   IOReturn result;
 
-  unsetInterface(device);
+  unsetInterface(devx);
 
   if (devx->deviceOpened) {
     result = (*devx->device)->USBDeviceClose(devx->device);
-    if (result != kIOReturnSuccess) setErrno(result, "USB device close");
+    if (result != kIOReturnSuccess) setUnixError(result, "USB device close");
     devx->deviceOpened = 0;
   }
 
@@ -896,7 +888,7 @@ usbFindDevice (UsbDeviceChooser chooser, void *data) {
     CFMutableDictionaryRef dictionary;
 
     if ((dictionary = IOServiceMatching(kIOUSBDeviceClassName))) {
-      io_iterator_t iterator = NULL;
+      io_iterator_t iterator = 0;
 
       kernelResult = IOServiceGetMatchingServices(port, dictionary, &iterator);
       dictionary = NULL;
@@ -913,7 +905,7 @@ usbFindDevice (UsbDeviceChooser chooser, void *data) {
                                                        kIOCFPlugInInterfaceID,
                                                        &plugin, &score);
           IOObjectRelease(service);
-          service = NULL;
+          service = 0;
 
           if ((ioResult == kIOReturnSuccess) && plugin) {
             IOUSBDeviceInterface182 **interface = NULL;
@@ -948,17 +940,17 @@ usbFindDevice (UsbDeviceChooser chooser, void *data) {
               (*interface)->Release(interface);
               interface = NULL;
             } else {
-              setErrno(ioResult, "USB device interface create");
+              setUnixError(ioResult, "USB device interface create");
             }
           } else {
-            setErrno(ioResult, "USB device service plugin create");
+            setUnixError(ioResult, "USB device service plugin create");
           }
         }
 
         IOObjectRelease(iterator);
-        iterator = NULL;
+        iterator = 0;
       } else {
-        setErrno(kernelResult, "USB device iterator create");
+        setUnixError(kernelResult, "USB device iterator create");
       }
     } else {
       LogPrint(LOG_ERR, "USB device matching dictionary create error.");
@@ -966,7 +958,7 @@ usbFindDevice (UsbDeviceChooser chooser, void *data) {
 
     mach_port_deallocate(mach_task_self(), port);
   } else {
-    setErrno(kernelResult, "Darwin master port create");
+    setUnixError(kernelResult, "Darwin master port create");
   }
 
   return device;
